@@ -8,8 +8,6 @@
 #
 ######################################################################
 
-from __future__ import absolute_import, print_function
-
 import argparse
 import datetime
 import functools
@@ -24,7 +22,6 @@ import signal
 import sys
 import time
 
-import six
 from class_registry import ClassRegistry
 
 from b2sdk.account_info.sqlite_account_info import (
@@ -36,6 +33,9 @@ from b2sdk.version import VERSION as b2sdk_version
 from b2sdk.v1 import (
     parse_sync_folder,
     AuthInfoCache,
+    B2Api,
+    B2Http,
+    B2RawApi,
     Synchronizer,
     SyncReport,
     NewerFileSyncMode,
@@ -50,7 +50,6 @@ from b2sdk.v1 import (
 from b2sdk.v1.exception import B2Error, BadFileInfo, MissingAccountData
 from b2.arg_parser import ArgumentParser, parse_comma_separated_list, \
     parse_millis_from_float_timestamp, parse_range
-from b2.cli_api import CliB2Api
 from b2.json_encoder import B2CliJsonEncoder
 from b2.version import VERSION
 
@@ -61,19 +60,28 @@ SEPARATOR = '=' * 40
 # Optional Env variable to use for getting account info while authorizing
 B2_APPLICATION_KEY_ID_ENV_VAR = 'B2_APPLICATION_KEY_ID'
 B2_APPLICATION_KEY_ENV_VAR = 'B2_APPLICATION_KEY'
-
-# Strings available to use when formatting doc strings.
-DOC_STRING_DATA = dict(
-    B2_ACCOUNT_INFO_ENV_VAR=B2_ACCOUNT_INFO_ENV_VAR,
-    B2_ACCOUNT_INFO_DEFAULT_FILE=B2_ACCOUNT_INFO_DEFAULT_FILE,
-    B2_APPLICATION_KEY_ID_ENV_VAR=B2_APPLICATION_KEY_ID_ENV_VAR,
-    B2_APPLICATION_KEY_ENV_VAR=B2_APPLICATION_KEY_ENV_VAR,
-)
+# Optional Env variable to use for adding custom string to the User Agent
+B2_USER_AGENT_APPEND_ENV_VAR = 'B2_USER_AGENT_APPEND'
 
 # Enable to get 0.* behavior in the command-line tool.
 # Disable for 1.* behavior.
 VERSION_0_COMPATIBILITY = False
 
+# The name of an executable entry point
+NAME = os.path.basename(sys.argv[0])
+if NAME.endswith('.py'):
+    NAME = 'b2'
+
+# Strings available to use when formatting doc strings.
+DOC_STRING_DATA = dict(
+    NAME=NAME,
+    B2_ACCOUNT_INFO_ENV_VAR=B2_ACCOUNT_INFO_ENV_VAR,
+    B2_ACCOUNT_INFO_DEFAULT_FILE=B2_ACCOUNT_INFO_DEFAULT_FILE,
+    B2_APPLICATION_KEY_ID_ENV_VAR=B2_APPLICATION_KEY_ID_ENV_VAR,
+    B2_APPLICATION_KEY_ENV_VAR=B2_APPLICATION_KEY_ENV_VAR,
+    B2_USER_AGENT_APPEND_ENV_VAR=B2_USER_AGENT_APPEND_ENV_VAR,
+)
+
 
 def current_time_millis():
     """
@@ -144,7 +152,7 @@ class Command(object):
         return decorator
 
     @classmethod
-    def get_parser(cls, subparsers=None, parents=None):
+    def get_parser(cls, subparsers=None, parents=None, disable_aliases=False):
         if parents is None:
             parents = []
 
@@ -161,7 +169,7 @@ class Command(object):
                 name,
                 description=cls.__doc__.format(**DOC_STRING_DATA),
                 parents=parents,
-                aliases=[alias] if alias is not None else ()
+                aliases=[alias] if alias is not None and not disable_aliases else ()
             )
 
         cls._setup_parser(parser)
@@ -179,7 +187,9 @@ class Command(object):
             subparsers = parser.add_subparsers(prog=parser.prog, title='usages', dest='command')
             subparsers.required = True
             for subcommand in cls.subcommands_registry.values():
-                subcommand.get_parser(subparsers, parents=parents)
+                subcommand.get_parser(
+                    subparsers=subparsers, parents=parents, disable_aliases=disable_aliases
+                )
 
         return parser
 
@@ -236,7 +246,7 @@ class B2(Command):
     file to use for caching authentication information.
     The default file to use is: {B2_ACCOUNT_INFO_DEFAULT_FILE}
 
-    For more details on one command: b2 <command> --help
+    For more details on one command: {NAME} <command> --help
 
     When authorizing with application keys, this tool requires that the key
     have the 'listBuckets' capability so that it can take the bucket names
@@ -244,10 +254,17 @@ class B2(Command):
     B2 Storage service.  Each different command may required additional
     capabilities.  You can find the details for each command in the help for
     that command.
+
+    A string provided via an optional environment variable {B2_USER_AGENT_APPEND_ENV_VAR}
+    will be appended to the User-Agent.
     """
 
     subcommands_registry = ClassRegistry()
 
+    @classmethod
+    def name_and_alias(cls):
+        return NAME, None
+
     def run(self, args):
         return self.subcommands_registry.get_class(args.command)
 
@@ -302,7 +319,7 @@ class AuthorizeAccount(Command):
         if args.applicationKeyId is None:
             args.applicationKeyId = (
                 os.environ.get(B2_APPLICATION_KEY_ID_ENV_VAR) or
-                six.moves.input('Backblaze application key ID: ')
+                input('Backblaze application key ID: ')
             )
 
         if args.applicationKey is None:
@@ -508,11 +525,11 @@ class CreateKey(Command):
 
     The capabilities are passed in as a comma-separated list, like "readFiles,writeFiles".
 
-    The 'validDurationSeconds' is the length of time the new application key will exist.
+    The 'duration' is the length of time the new application key will exist.
     When the time expires the key will disappear and will no longer be usable.  If not
     specified, the key will not expire.
 
-    The 'bucketName' is the name of a bucket in the account.  When specified, the key
+    The 'bucket' is the name of a bucket in the account.  When specified, the key
     will only allow access to that bucket.
 
     The 'namePrefix' restricts file access to files whose names start with the prefix.
@@ -890,13 +907,14 @@ class ListKeys(Command):
     Lists the application keys for the current account.
 
     The columns in the output are:
-        - ID of the application key
-        - Name of the application key
-        - Name of the bucket the key is restricted to, or '-' for no restriction
-        - Date of expiration, or '-'
-        - Time of expiration, or '-'
-        - File name prefix, in single quotes
-        - Command-separated list of capabilities
+
+    - ID of the application key
+    - Name of the application key
+    - Name of the bucket the key is restricted to, or '-' for no restriction
+    - Date of expiration, or '-'
+    - Time of expiration, or '-'
+    - File name prefix, in single quotes
+    - Command-separated list of capabilities
 
     None of the values contain whitespace.
 
@@ -1012,9 +1030,8 @@ class ListUnfinishedLargeFiles(Command):
     def run(self, args):
         bucket = self.api.get_bucket_by_name(args.bucketName)
         for unfinished in bucket.list_unfinished_large_files():
-            file_info_text = six.u(' ').join(
-                '%s=%s' % (k, unfinished.file_info[k])
-                for k in sorted(six.iterkeys(unfinished.file_info))
+            file_info_text = ' '.join(
+                '%s=%s' % (k, unfinished.file_info[k]) for k in sorted(unfinished.file_info)
             )
             self._print(
                 '%s %s %s %s' %
@@ -1055,7 +1072,6 @@ class Ls(Command):
         parser.add_argument('--json', action='store_true')
         parser.add_argument('--versions', action='store_true')
         parser.add_argument('--recursive', action='store_true')
-        parser.add_argument('--prefix', action='store_true')
         parser.add_argument('bucketName')
         parser.add_argument('folderName', nargs='?')
 
@@ -1128,6 +1144,15 @@ class Sync(Command):
     Copies multiple files from source to destination.  Optionally
     deletes or hides destination files that the source does not have.
 
+    The synchronizer can copy files:
+
+    - From a B2 bucket to a local destination.
+    - From a local source to a B2 bucket.
+    - From one B2 bucket to another.
+    - Between different folders in the same B2 bucket.
+
+    Use "b2://<bucketName>/<prefix>" for B2 paths, e.g. "b2://my-bucket-name/a/path/prefix/".
+
     Progress is displayed on the console unless '--noProgress' is
     specified.  A list of actions taken is always printed.
 
@@ -1209,10 +1234,6 @@ class Sync(Command):
     not be synced. Specifying --verbose and --dryRun can be useful to
     determine comparison value differences.
 
-    One of the paths must be a local file path, and the other must be
-    a B2 bucket path. Use "b2://<bucketName>/<prefix>" for B2 paths, e.g.
-    "b2://my-bucket-name/a/path/prefix/".
-
     When a destination file is present that is not in the source, the
     default is to leave it there.  Specifying --delete means to delete
     destination files that are not in the source.
@@ -1230,7 +1251,7 @@ class Sync(Command):
     old file from the source will replace the newer one in the destination.
 
     To make the destination exactly match the source, use:
-        b2 sync --delete --replaceNewer ... ...
+    {NAME} sync --delete --replaceNewer ... ...
 
     WARNING: Using '--delete' deletes files!  We recommend not using it.
     If you use --keepDays instead, you will have some time to recover your
@@ -1238,11 +1259,11 @@ class Sync(Command):
 
     To make the destination match the source, but retain previous versions
     for 30 days:
-        b2 sync --keepDays 30 --replaceNewer ... b2://...
+    {NAME} sync --keepDays 30 --replaceNewer ... b2://...
 
     Example of sync being used with excludeRegex. This will ignore .DS_Store files
     and .Spotlight-V100 folders
-        b2 sync -excludeRegex '(.*\.DS_Store)|(.*\.Spotlight-V100)' ... b2://...
+    {NAME} sync -excludeRegex '(.*\.DS_Store)|(.*\.Spotlight-V100)' ... b2://...
 
     Requires capabilities: listFiles, readFiles (for downloading), writeFiles (for uploading)
     """
@@ -1500,7 +1521,7 @@ class ConsoleTool(object):
             return command.run(args)
         except MissingAccountData as e:
             logger.exception('ConsoleTool missing account data error')
-            self._print_stderr('ERROR: %s  Use: b2 authorize-account' % (str(e),))
+            self._print_stderr('ERROR: %s  Use: %s authorize-account' % (str(e), NAME))
             return 1
         except B2Error as e:
             logger.exception('ConsoleTool command error')
@@ -1526,7 +1547,7 @@ class ConsoleTool(object):
         self._print('File size:   ', download_dest.content_length)
         self._print('Content type:', download_dest.content_type)
         self._print('Content sha1:', download_dest.content_sha1)
-        for name in sorted(six.iterkeys(download_dest.file_info)):
+        for name in sorted(download_dest.file_info):
             self._print('INFO', name + ':', download_dest.file_info[name])
         if download_dest.content_sha1 != 'none':
             self._print('checksum matches')
@@ -1569,20 +1590,7 @@ class ConsoleTool(object):
 
 
 # used by Sphinx
-get_parser = B2.get_parser
-
-
-def decode_sys_argv():
-    """
-    Returns the command-line arguments as unicode strings, decoding
-    whatever format they are in.
-
-    https://stackoverflow.com/questions/846850/read-unicode-characters-from-command-line-arguments-in-python-2-x-on-windows
-    """
-    if six.PY2:
-        encoding = sys.getfilesystemencoding()
-        return [arg.decode(encoding) for arg in sys.argv]
-    return sys.argv
+get_parser = functools.partial(B2.get_parser, disable_aliases=True)
 
 
 # TODO: import from b2sdk as soon as we rely on 1.0.0
@@ -1607,10 +1615,10 @@ class InvalidArgument(B2Error):
 def main():
     info = SqliteAccountInfo()
     cache = AuthInfoCache(info)
-    b2_api = CliB2Api(info, cache=cache)
+    raw_api = B2RawApi(B2Http(user_agent_append=os.environ.get(B2_USER_AGENT_APPEND_ENV_VAR)))
+    b2_api = B2Api(info, cache=cache, raw_api=raw_api)
     ct = ConsoleTool(b2_api=b2_api, stdout=sys.stdout, stderr=sys.stderr)
-    decoded_argv = decode_sys_argv()
-    exit_status = ct.run_command(decoded_argv)
+    exit_status = ct.run_command(sys.argv)
     logger.info('\\\\ %s %s %s //', SEPARATOR, ('exit=%s' % exit_status).center(8), SEPARATOR)
 
     # I haven't tracked down the root cause yet, but in Python 2.7, the futures
